// bstickerproxy.js
//
// BSTickerProxy is a helper object that wraps a native Brightsign ticker object and
// a place-holder div in order to provide auto-close behavior and make page
// layout easier.
//
// 1. The native ticker does not resizing without re-creating the widget. The position
//    maybe be changed via SetRectangle but if the width and height has changed then the
//    animation will be reset.
// 2. By default BSTickerProxy registers listeners for the window 'beforeunload' event
//    and the place-holder div's 'DOMNodeRemoved' events and automatically calls BSTicker.Shutdown()
//    to ensure that the BSTicker is not orphaned.
//    This can be overriden by setting the noautoclose property to true.
// 3. SetForegroundColor and SetBackgroundColor use HTML/CSS colors.
// 4. The ticker scroll direction is determined examining the language (unicode characters) of
//    the first element. It may be necessary to set the meta charset attribute in the HTML header.
// 5. The font face may either be set by calling SetFont with the file name e.g. "sd:comic.ttf" OR
//    the font-face can be specified or derived from the CSS font.
//    SetFont("font-face: DejaVu Sans,Sans Italic Bold")
//
// ticker_parameters {
//   id: DOMString,        // Required: The id of the place-holder div
//   layout: {             // Optional: If null then native ticker will use CSS layout of place-holder div
//      x: Integer,        // the absolute screen x-coordinate in pixels
//      y: Integer,        // the absolute screen y-coordinate in pixels
//      w: Integer,        // the absolute screen width in pixels
//      h: Integer,        // the absolute screen height in pixels
//      r: Integer,        // the rotation of the text degrees (0, 90, 180, 270) - default 0
//   }
//   noautoclose: boolean, // Optional: Default false - If true, disable auto close
//   debug: boolean        // Optional: Default false - debug trace to console
// }
function BSTickerProxy(ticker_parameters) {
    var bs_ticker;
    var element;
    var id;
    var strings;
    var params;
    var x;
    var y;
    var w;
    var h;
    var r;
    var foreground_color;
    var background_color;
    var separator_string;
    var separator;
    var pps;
    var hidden;
    var before_unload_listener;
    var node_removed_listener;
    var debug_enabled = false;
    var font_face;
    var autopop;
    var rightmost;

    Init(ticker_parameters);

    function debug(function_name, str) {
        if (debug_enabled)
            console.log('BSTickerProxy:' + function_name + ': ' + str);
    }

    function Init(p) {
        if (! p)
            throw('BSTickerProxy: null parameters');

        // Init may be called to reset a ticker so Shutdown first
        Shutdown();
        x = y = w = h = r = 0;
        foreground_color = 'white';
        background_color = 'black';
        separator_string = '';
        separator = 'diamond';
        pps = 0;
        hidden = false;
        strings = [];
        params = p;
        debug_enabled = false;
        autopop = false;

        id = params.id;
        element = document.getElementById(id);
        if (! element)
            throw('Missing ticker element: ' + id);

        if (params['debug'])
            debug_enabled = true;

        getLayout();

        if (! params['noautoclose']) {
            before_unload_listener = Shutdown.bind(this);
            window.addEventListener('beforeunload', before_unload_listener);
            node_removed_listener = nodeRemoved.bind(this);
            element.addEventListener('DOMNodeRemoved', node_removed_listener);
        }

        if (navigator.userAgent.match(/BrightSign/)) {
            if (!bs_ticker) {
                debug('Init', '(' + x + ',' + y + ',' + w + ',' + h + ',' + r + ')');
                bs_ticker = new BSTicker(x, y, w, h, r);
            }

            if (!bs_ticker)
                throw('Failed to create BSTicker: ' + navigator.userAgent);

            set_css_layout();
            SetForegroundColor(foreground_color);
            SetBackgroundColor(background_color);
            if (font_face)
                SetFont("font-face:" + font_face);
        }
        else if (! params) {
            // Not a BrightScript ticker but layout params still defined
            // so do explicit CSS update.
            set_css_layout();
            element.style.color = foreground_color;
            element.style.backgroundColor = background_color;
        }
    }

    function Shutdown() {
        if (element)
            debug('shutdown', element);


        if (bs_ticker) {
            bs_ticker.ontickerstringremoved = null;
            bs_ticker.Shutdown();
        }
        bs_ticker = null;

        if (before_unload_listener)
            window.removeEventListener('beforeunload', before_unload_listener);
        before_unload_listener = null;

        if (element && node_removed_listener)
            element.removeEventListener('DOMNodeRemoved', node_removed_listener);
        node_removed_listener = null;

        element = null;
    }

    function nodeRemoved() {
        debug('nodeRemoved closing ticker', bs_ticker);
        Shutdown();
    }

    function is_numeric(a) {
        return ! isNaN(a);
    }

    // If an explicit layout is defined then set the div
    // to be the same as the ticker layout parameters.
    // Otherwise, use the browser layout to set the ticker
    // parameters.
    function getLayout() {
        if (params.layout) {
            debug('getLayout', 'using explicit layout params');
            if (! is_numeric(params.layout.x))
                throw('BSTickerProxy: Layout has invalid/missing x property');
            x = params.layout.x;

            if (! is_numeric(params.layout.y))
                throw('BSTickerProxy: Layout has invalid/missing y property');
            y = params.layout.y;

            if (! is_numeric(params.layout.w))
                throw('BSTickerProxy: Layout has invalid/missing w property');
            w = params.layout.w;

            if (! is_numeric(params.layout.h))
                throw('BSTickerProxy: Layout has invalid/missing h property');
            h = params.layout.h;

            if (! is_numeric(params.layout.r))
                r = 0;
            else
                r = params.layout.r;

            element.style.position = 'fixed';
            hidden = false;
        }
        else {
            debug('getLayout', 'using CSS layout');
            var clientRect = element.getBoundingClientRect();
            debug('getLayout', clientRect.left + ',' + clientRect.top + ',' + clientRect.width + ',' + clientRect.height);
            x = clientRect.left;
            y = clientRect.top;
            w = clientRect.width;
            h = clientRect.height;
            if (window.getComputedStyle(element).display == 'none' || window.getComputedStyle(element).visibility == 'hidden')
                hidden = true;
            else
                hidden = false;
            foreground_color = window.getComputedStyle(element).color;
            background_color = window.getComputedStyle(element).backgroundColor;

            var family = window.getComputedStyle(element).fontFamily;
            if (family) {
                font_face = family.replace(/'/g, "");

                var style = window.getComputedStyle(element).fontStyle;
                if (style == 'italic')
                    font_face += ' Italic';
                else if (style == 'Oblique')
                    font_face += ' Oblique';

                var weight = window.getComputedStyle(element).fontWeight;
                if (weight == 'bold')
                    font_face += ' Bold';

                debug('getLayout', 'font-face: ' + font_face);
            }
            else {
                font_face = null;
            }

        }
    }

    // Update or re-apply the CSS layout
    function set_css_layout() {
        if (element) {
            element.style.width = w + 'px';
            element.style.height = h + 'px';
            element.style.left = x + 'px';
            element.style.top = y + 'px';
        }
    }

    // Returns the native BSTicker
    function GetBsTicker() {
        return bs_ticker;
    }

    // Returns the string_id (integer) which may be used to identify
    // which string has been removed when the TickerStringRemoved event
    // is generated.
    function AddString(str) {
        var ret = 1
        debug('AddString: ', str);
        strings.push(str);
        if (bs_ticker)
            ret = bs_ticker.AddString(str);
        return ret;
    }

    function PopStrings(count) {
        var ret = true;
        debug('PopStrings', count);
        for (var i = 0; i < count; i++) {
            var s = strings.shift();
            debug('PopStrings', s);
        }

        if (bs_ticker)
            ret = bs_ticker.PopStrings(count);
        return ret;
    }

    function PopAllStrings(){
        debug('PopAllStrings', "");
        PopStrings(strings.length)
    }

    // Return components as array [r,g,b,a]
    function resolve_color_components(color) {
        var ret = [ 0, 0, 0 ];
        var color_div = document.createElement('div');
        color_div.name = 'color_div';
        color_div.display = 'none';
        document.body.appendChild(color_div);

        color_div.style.color = color;
        var style_color = window.getComputedStyle(color_div).color;
        var style_color_components = style_color.replace(/[a-z|(),]*/g, '').split(' '); // Extract the component values
        var col_val = 0x0;
        for (var i = 0; i < ret.length; i++)
            ret[i] = Number(style_color_components[i]);

        if (style_color_components.length == 4)
            ret.push(Math.round(style_color_components[3] * 255));
        else
            ret.push(255);

        debug('resolve_color_components', color + '->' + style_color_components + '->' + ret);
        document.body.removeChild(color_div);
        return ret;
    }

    function SetForegroundColor(color) {
        var ret = true;
        debug('SetForegroundColor', color);
        foreground_color = color;
        var comp = resolve_color_components(color);
        if (bs_ticker)
            ret = bs_ticker.SetForegroundColor(
                    comp[3], // a
                    comp[0], // r
                    comp[1], // g
                    comp[2]  // b
                    );
        else
            if (element)
                element.style.color = color;
        return ret;
    }

    function SetBackgroundColor(color) {
        var ret = true;
        debug('SetBackgroundColor', color);
        background_color = color;
        comp = resolve_color_components(color);
        if (bs_ticker)
            ret = bs_ticker.SetBackgroundColor(
                    comp[3], // a
                    comp[0], // r
                    comp[1], // g
                    comp[2]  // b
                    );
        else
            if (element)
                element.style.backgroundColor = color;
        return ret;
    }

    function SetSeparatorString(sep) {
        var ret = true;
        debug('SetSeparatorString', sep);
        separator = 'string';
        separator_string = sep;
        if (bs_ticker)
            ret = bs_ticker.SetSeparatorString(separator_string);
        return ret;
    }

    function SetSeparatorCircle() {
        var ret = true;
        debug('SetSeparatorCircle', '');
        separator = 'circle';
        if (bs_ticker)
            ret = bs_ticker.SetSeparatorCircle();
        return ret;
    }

    function SetSeparatorDiamond() {
        var ret = true;
        debug('SetSeparatorDiamond', '');
        separator = 'diamond';
        if (bs_ticker)
            ret = bs_ticker.SetSeparatorDiamond();
        return ret;
    }

    function SetSeparatorSquare() {
        var ret = true;
        debug('SetSeparatorSquare', '');
        separator = 'square';
        if (bs_ticker)
            ret = bs_ticker.SetSeparatorSquare();
        return ret;
    }

    function SetPixelsPerSecond(pixels_per_second) {
        var ret = true;
        var pps = pixels_per_second;
        debug('SetPixelsPerSecond', pixels_per_second);
        if (bs_ticker)
            ret = bs_ticker.SetPixelsPerSecond(pps);
        return ret;
    }

    function SetFont(font_face) {
        var ret = true;
        this.font_face = font_face;
        debug('SetFont', this.font_face);
        if (bs_ticker)
            ret = bs_ticker.SetFont(this.font_face);
        return ret;
    }

    function SetMultiscreen(offset, size, ip_address, port) {
        var ret = true;
        debug('SetMultiscreen', '' + offset + ' ' + size + ' ' + ip_address + ' ' + port);
        if (bs_ticker)
            ret = bs_ticker.SetMultiscreen(offset, size, ip_address, port);
        else
            console.log('SetMultiscreen not supported in HTML mode');
        return ret;
    }

    function SetAutoPop(enable) {
        var ret = true;
        var autopop = Boolean(enable);
        debug('SetAutoPop', autopop);
        if (bs_ticker)
            ret = bs_ticker.SetAutoPop(enable);
        return ret;
    }

    function SetRectangle(x, y, w, h) {
        var ret = true;
        //debug('SetRectangle', '(' + x + ',' + y + ',' + w + ',' + h);
        this.x = x;
        this.y = y;
        this.w = w;
        this.h = h;
        if (bs_ticker)
            ret = bs_ticker.SetRectangle(x,y,w,h);
        set_css_layout();
    }

    function SetRightmost(enable){
        debug('SetRightmost',enable);
        rightmost = enable;
    }

    function GetRightmost(){
        debug('GetRightmost', this.rightmost);
        return rightmost;
    }

    function SetTickerStringRemovedListener(listener) {
        debug('SetTickerStringRemovedListener', listener);
        if (bs_ticker)
            bs_ticker.ontickerstringremoved = listener;
    }

    return {
        GetBsTicker: GetBsTicker,
        AddString: AddString,
        PopStrings: PopStrings,
        PopAllStrings: PopAllStrings,
        SetForegroundColor: SetForegroundColor,
        SetBackgroundColor: SetBackgroundColor,

        SetSeparatorString: SetSeparatorString,
        SetSeparatorCircle: SetSeparatorCircle,
        SetSeparatorDiamond: SetSeparatorDiamond,
        SetSeparatorSquare: SetSeparatorSquare,

        SetPixelsPerSecond: SetPixelsPerSecond,
        SetMultiscreen: SetMultiscreen,
        SetFont: SetFont,
        SetRectangle: SetRectangle,
        SetRightmost: SetRightmost,
        GetRightmost: GetRightmost,

        SetAutoPop: SetAutoPop,
        SetTickerStringRemovedListener: SetTickerStringRemovedListener,

        Init: Init,
        Shutdown: Shutdown
    }
}
